"use strict";
// cspell:words nothrow
Object.defineProperty(exports, "__esModule", { value: true });
exports.createSpawn = void 0;
const Logger_1 = require("./Logger");
const zx_1 = require("zx");
const log = Logger_1.Logger.child({
    namespace: 'createSpawn',
});
const prefixLines = (subject, prefix) => {
    const response = [];
    for (const fragment of subject.split('\n')) {
        response.push(prefix + fragment);
    }
    return response.join('\n');
};
const createSpawn = (taskId, triggerSignal) => {
    return async (pieces, ...args) => {
        // eslint-disable-next-line promise/prefer-await-to-then
        const processPromise = (0, zx_1.$)(pieces, ...args)
            .nothrow()
            .quiet();
        (async () => {
            for await (const chunk of processPromise.stdout) {
                // eslint-disable-next-line no-console
                console.log(prefixLines(chunk.toString().trimEnd(), taskId + ' > '));
            }
        })();
        (async () => {
            for await (const chunk of processPromise.stderr) {
                // eslint-disable-next-line no-console
                console.error(prefixLines(chunk.toString().trimEnd(), taskId + ' > '));
            }
        })();
        if (triggerSignal) {
            const kill = () => {
                processPromise.kill();
            };
            triggerSignal.addEventListener('abort', kill, {
                once: true,
            });
            // eslint-disable-next-line promise/prefer-await-to-then
            processPromise.finally(() => {
                triggerSignal.removeEventListener('abort', kill);
            });
        }
        const result = await processPromise;
        if (result.exitCode === 0) {
            return result;
        }
        if (triggerSignal === null || triggerSignal === void 0 ? void 0 : triggerSignal.aborted) {
            return result;
        }
        log.error('task %s exited with an error', taskId);
        throw new Error('Program exited with code ' + result.exitCode + '.');
    };
};
exports.createSpawn = createSpawn;
