"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const subscribe_1 = require("./subscribe");
const node_timers_1 = require("node:timers");
const sinon = __importStar(require("sinon"));
const vitest_1 = require("vitest");
const defaultTrigger = {
    expression: ['match', 'foo', 'basename'],
    id: 'foo',
    interruptible: false,
    name: 'foo',
    onChange: async () => { },
    relativePath: 'foo',
    retry: {
        retries: 0,
    },
    watch: 'foo',
};
const wait = (time) => {
    return new Promise((resolve) => {
        (0, node_timers_1.setTimeout)(resolve, time);
    });
};
(0, vitest_1.it)('rejects promise if Watchman "subscribe" command produces an error', async () => {
    const client = {
        command: () => { },
    };
    const trigger = {
        ...defaultTrigger,
    };
    const clientMock = sinon.mock(client);
    clientMock
        .expects('command')
        .once()
        .callsFake((args, callback) => {
        callback(new Error('foo'));
    });
    await (0, vitest_1.expect)((0, subscribe_1.subscribe)(client, trigger)).rejects.toThrowError('foo');
    (0, vitest_1.expect)(clientMock.verify());
});
(0, vitest_1.it)('evaluates onChange', async () => {
    const client = {
        command: () => { },
        on: () => { },
    };
    const trigger = {
        ...defaultTrigger,
    };
    const subscriptionMock = sinon.mock(trigger);
    const abortController = new AbortController();
    subscriptionMock
        .expects('onChange')
        .once()
        .callsFake(() => {
        abortController.abort();
        return Promise.resolve(null);
    });
    const clientMock = sinon.mock(client);
    clientMock
        .expects('on')
        .once()
        .callsFake((event, callback) => {
        callback({
            files: [],
            subscription: 'foo',
        });
    });
    await (0, subscribe_1.subscribe)(client, trigger, abortController.signal);
    (0, vitest_1.expect)(clientMock.verify());
    (0, vitest_1.expect)(subscriptionMock.verify());
});
(0, vitest_1.it)('evaluates multiple onChange', async () => {
    const client = {
        command: () => { },
        on: () => { },
    };
    const trigger = {
        ...defaultTrigger,
    };
    const subscriptionMock = sinon.mock(trigger);
    const onChange = subscriptionMock.expects('onChange').thrice();
    const abortController = new AbortController();
    onChange.onFirstCall().resolves(null);
    onChange.onSecondCall().resolves(null);
    onChange.onThirdCall().callsFake(() => {
        abortController.abort();
        return Promise.resolve(null);
    });
    const clientMock = sinon.mock(client);
    clientMock.expects('on').callsFake((event, callback) => {
        callback({
            files: [],
            subscription: 'foo',
        });
        (0, node_timers_1.setTimeout)(() => {
            callback({
                files: [],
                subscription: 'foo',
            });
            (0, node_timers_1.setTimeout)(() => {
                callback({
                    files: [],
                    subscription: 'foo',
                });
            });
        });
    });
    await (0, subscribe_1.subscribe)(client, trigger, abortController.signal);
    (0, vitest_1.expect)(onChange.callCount).toBe(3);
});
(0, vitest_1.it)('debounces onChange', async () => {
    const client = {
        command: () => { },
        on: () => { },
    };
    const trigger = {
        ...defaultTrigger,
        debounce: {
            wait: 100,
        },
    };
    const subscriptionMock = sinon.mock(trigger);
    const onChange = subscriptionMock.expects('onChange').thrice();
    const abortController = new AbortController();
    (0, node_timers_1.setTimeout)(() => {
        abortController.abort();
    }, 200);
    onChange.onFirstCall().resolves(null);
    const clientMock = sinon.mock(client);
    clientMock.expects('on').callsFake((event, callback) => {
        callback({
            files: [],
            subscription: 'foo',
        });
        (0, node_timers_1.setTimeout)(() => {
            callback({
                files: [],
                subscription: 'foo',
            });
            (0, node_timers_1.setTimeout)(() => {
                callback({
                    files: [],
                    subscription: 'foo',
                });
            });
        });
    });
    await (0, subscribe_1.subscribe)(client, trigger, abortController.signal);
    (0, vitest_1.expect)(onChange.callCount).toBe(1);
});
(0, vitest_1.it)('waits for onChange to complete when { interruptible: false }', async () => {
    const client = {
        command: () => { },
        on: () => { },
    };
    const trigger = {
        ...defaultTrigger,
        interruptible: false,
    };
    const abortController = new AbortController();
    const triggerMock = sinon.mock(trigger);
    const onChange = triggerMock.expects('onChange').twice();
    let completed = false;
    onChange.onFirstCall().callsFake(async () => {
        await wait(100);
        completed = true;
    });
    onChange.onSecondCall().callsFake(() => {
        (0, vitest_1.expect)(completed).toBe(true);
        abortController.abort();
    });
    const clientMock = sinon.mock(client);
    clientMock.expects('on').callsFake((event, callback) => {
        callback({
            files: [],
            subscription: 'foo',
        });
        callback({
            files: [],
            subscription: 'foo',
        });
    });
    await (0, subscribe_1.subscribe)(client, trigger, abortController.signal);
    (0, vitest_1.expect)(onChange.callCount).toBe(2);
});
(0, vitest_1.it)('throws if onChange produces an error', async () => {
    const client = {
        command: () => { },
        on: () => { },
    };
    const trigger = {
        ...defaultTrigger,
    };
    const subscriptionMock = sinon.mock(trigger);
    subscriptionMock.expects('onChange').rejects(new Error('foo'));
    const abortController = new AbortController();
    const clientMock = sinon.mock(client);
    clientMock.expects('on').callsFake((event, callback) => {
        callback({
            files: [],
            subscription: 'foo',
        });
    });
    await (0, vitest_1.expect)((0, subscribe_1.subscribe)(client, trigger, abortController.signal)).rejects.toThrowError('foo');
    await abortController.abort();
});
(0, vitest_1.it)('retries failing routines', async () => {
    const client = {
        command: () => { },
        on: () => { },
    };
    const trigger = {
        ...defaultTrigger,
        retry: {
            retries: 1,
        },
    };
    const subscriptionMock = sinon.mock(trigger);
    const onChange = subscriptionMock.expects('onChange').twice();
    const abortController = new AbortController();
    onChange.onFirstCall().rejects(new Error('foo'));
    onChange.onSecondCall().callsFake(() => {
        abortController.abort();
        return Promise.resolve(null);
    });
    const clientMock = sinon.mock(client);
    clientMock.expects('on').callsFake((event, callback) => {
        callback({
            files: [],
            subscription: 'foo',
        });
    });
    await (0, subscribe_1.subscribe)(client, trigger, abortController.signal);
});
(0, vitest_1.it)('reports { first: true } only for the first event', async () => {
    const client = {
        command: () => { },
        on: () => { },
    };
    const trigger = {
        ...defaultTrigger,
    };
    const subscriptionMock = sinon.mock(trigger);
    const onChange = subscriptionMock.expects('onChange').twice();
    onChange.onFirstCall().resolves(null);
    const abortController = new AbortController();
    onChange.onSecondCall().callsFake(() => {
        abortController.abort();
        return Promise.resolve(null);
    });
    const clientMock = sinon.mock(client);
    clientMock.expects('on').callsFake((event, callback) => {
        callback({
            files: [],
            subscription: 'foo',
        });
        callback({
            files: [],
            subscription: 'foo',
        });
    });
    await (0, subscribe_1.subscribe)(client, trigger, abortController.signal);
    (0, vitest_1.expect)(onChange.args).toMatchObject([
        [
            {
                first: true,
            },
        ],
        [
            {
                first: false,
            },
        ],
    ]);
    (0, vitest_1.expect)(subscriptionMock.verify());
});
